/////////////////////////////////////////////////////////////////////////////
// GdiTest.c
//
// Main window source for GDI example.
//
// Copyright (C) 1999, Nohau Corporation
// All rights reserved.

#include "Windows.h"

#define GDI_USER_MAIN
#include "GdiUser.h"

#include "GdiExample.h"

// Local routines in this file
static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
static BOOL InitializeGdiPointers(HINSTANCE hDLL);
static void CreateMainWindow(UINT);
static void OnCommand(HWND, int);

// Handle to main application window
HWND g_hMainWnd = NULL;

// The main window displays these output lines set by command code
char* g_outputLines[MAX_NUM_OUTPUT_LINES];

// Other globals
HINSTANCE g_hInstance;

DiRegisterInfoT g_registers[NUM_REGISTERS] = {
    // ID 0 used for program counter
    {"SP",          1, FALSE},
    {"Z N V VT",    2, FALSE},
    {"C P I ST",    3, FALSE},
    {"IMASK",       4, FALSE},
    {"IMASK1",      5, FALSE},
    {"WSR",         6, FALSE},
    {"FRAM01",      7, FALSE},
    {"Tmp0",        8, FALSE},
    {"Tmp1",        9, FALSE},
    {"Tmp2",        10, FALSE}
};

DiMemorySpaceInfoT g_memorySpaces[NUM_MEMORY_SPACES] = {

    {"CODE",        0, FALSE},
    {"CODE FAR",    1, FALSE},
    {"DATA",        2, FALSE},
    {"DATA FAR",    3, FALSE},
    {"EMUL MEM",    4, FALSE},
    {"BREAK MAP",   5, FALSE},
    {"SHADOW",      6, FALSE},
    {"TRACE",       7, FALSE}
};

BreakpointInfo g_breakpoints[MAX_NUM_BREAKPOINTS];
int g_nBreakpoints = 0;

/////////////////// WinMain()

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, 
				   PSTR pszCmdLine, int fCmdShow)
{
	MSG msg;
    DiReturnT fRet;
    HINSTANCE hDLL;
    int nTemp = 0;
    DiUInt32T nVersion;
    DiFeaturesT features;

	// Initialize globals
	memset(g_outputLines, 0, sizeof(g_outputLines));
	g_hInstance = hInstance;

    // Set working directory to Nohau\Seehau
    if (!SetCurrentDirectory("c:\\Nohau\\Seehau196"))
    {
        MessageBox(NULL, "Incorrect Seehau directory path.", "Error", MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Load core DLL
    hDLL = LoadLibrary(EMUL_DLL_NAME);
    if (hDLL == NULL)
    {
        MessageBox(NULL, "Unable to load emulator DLL.", "Error", MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Initialize GDI function pointers
    if (!InitializeGdiPointers(hDLL))
    {
        MessageBox(NULL, "DLL does not support all GDI functions.", "Error", 
            MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Initialize GDI (no command line args, no GUI callback)
    fRet = DiGdiOpen(GDI_VERSION_HIGH, GDI_VERSION_LOW, 0, NULL, NULL);
    if (fRet != DI_OK)
    {
        MessageBox(NULL, "DiGdiOpen failed.", "Error", MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Get version supported by core
    DiGdiVersion(&nVersion);

    // Display version
    g_outputLines[0] = malloc(256);
    sprintf(g_outputLines[0], "GDI version = 0x%x", nVersion);

    // Get features supported
    DiGdiGetFeatures(&features);

    // Set mapping of symbolic register names to ID numbers
    fRet = DiGdiInitRegisterMap(NUM_REGISTERS, g_registers, PC_ID);
    if (fRet != DI_OK)
    {
        MessageBox(NULL, "DiGdiInitRegisterMap failed.", "Error", 
            MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Set names of memory spaces also
    fRet = DiGdiInitMemorySpaceMap(NUM_MEMORY_SPACES, g_memorySpaces);
    if (fRet != DI_OK)
    {
        MessageBox(NULL, "DiGdiInitMemorySpaceMap failed.", "Error", 
            MB_OK|MB_ICONSTOP);
        return 0;
    }

    // Create main window
	CreateMainWindow(fCmdShow);

    // Run message loop
	while (GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

    // Cleanup
    DiGdiClose(TRUE);
    FreeOutputLines();

	return msg.wParam;
}

/////////////////// InitializeGdiPointers()
//
// Initializes function pointers to GDI routines in core DLL.

BOOL InitializeGdiPointers(HINSTANCE hDLL)
{
    g_pfnDiGdiOpen = (DiGdiOpenPtr) GetProcAddress(hDLL, "DiGdiOpen");
    g_pfnDiGdiClose = (DiGdiClosePtr) GetProcAddress(hDLL, "DiGdiClose");
    g_pfnDiGdiVersion = (DiGdiVersionPtr) GetProcAddress(hDLL, "DiGdiVersion");
    g_pfnDiGdiGetFeatures = (DiGdiGetFeaturesPtr) GetProcAddress(hDLL, "DiGdiGetFeatures");
    g_pfnDiGdiInitRegisterMap = (DiGdiInitRegisterMapPtr) GetProcAddress(hDLL, "DiGdiInitRegisterMap");
    g_pfnDiGdiInitMemorySpaceMap = (DiGdiInitMemorySpaceMapPtr) GetProcAddress(hDLL, "DiGdiInitMemorySpaceMap");
    g_pfnDiRegisterRead = (DiRegisterReadPtr) GetProcAddress(hDLL, "DiRegisterRead");
    g_pfnDiRegisterWrite = (DiRegisterWritePtr) GetProcAddress(hDLL, "DiRegisterWrite");
    g_pfnDiMemoryRead = (DiMemoryReadPtr) GetProcAddress(hDLL, "DiMemoryRead");
    g_pfnDiMemoryWrite = (DiMemoryWritePtr) GetProcAddress(hDLL, "DiMemoryWrite");
    g_pfnDiExecResetChild = (DiExecResetChildPtr) GetProcAddress(hDLL, "DiExecResetChild");
    g_pfnDiExecSingleStep = (DiExecSingleStepPtr) GetProcAddress(hDLL, "DiExecSingleStep");
    g_pfnDiExecContinue = (DiExecContinuePtr) GetProcAddress(hDLL, "DiExecContinue");
    g_pfnDiExecContinueUntil = (DiExecContinueUntilPtr) GetProcAddress(hDLL, "DiExecContinueUntil");
    g_pfnDiExecStop = (DiExecStopPtr) GetProcAddress(hDLL, "DiExecStop");
    g_pfnDiBreakpointSet = (DiBreakpointSetPtr) GetProcAddress(hDLL, "DiBreakpointSet");
    g_pfnDiBreakpointClear = (DiBreakpointClearPtr) GetProcAddress(hDLL, "DiBreakpointClear");
    g_pfnDiBreakpointClearAll = (DiBreakpointClearAllPtr) GetProcAddress(hDLL, "DiBreakpointClearAll");
    g_pfnDiExecGetStatus = (DiExecGetStatusPtr) GetProcAddress(hDLL, "DiExecGetStatus");

    if (!g_pfnDiGdiOpen
        || !g_pfnDiGdiClose
        || !g_pfnDiGdiVersion
        || !g_pfnDiGdiGetFeatures
        || !g_pfnDiGdiInitRegisterMap
        || !g_pfnDiGdiInitMemorySpaceMap
        || !g_pfnDiRegisterRead
        || !g_pfnDiRegisterWrite
        || !g_pfnDiMemoryRead
        || !g_pfnDiMemoryWrite
        || !g_pfnDiExecResetChild
        || !g_pfnDiExecSingleStep
        || !g_pfnDiExecContinue
        || !g_pfnDiExecContinueUntil
        || !g_pfnDiExecStop
        || !g_pfnDiBreakpointSet
        || !g_pfnDiBreakpointClear
        || !g_pfnDiBreakpointClearAll
        || !g_pfnDiExecGetStatus)
    {
        return FALSE;
    }

    return TRUE;
}

/////////////////// FreeOutputLines()
//
// Frees output lines buffer.

void FreeOutputLines()
{
    int i;
    for (i = 0; i < MAX_NUM_OUTPUT_LINES && g_outputLines[i]; i++)
    {
        free(g_outputLines[i]);
        g_outputLines[i] = NULL;
    }
}

/////////////////// WndProc()
//
// Window procedure for main window.

static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HDC	hDC;
	PAINTSTRUCT ps;
	HFONT hOldFont;
	TEXTMETRIC tm;
	int i, ny;

	switch (uMsg)
	{
	case WM_PAINT:

		// Select fixed width font
		hDC = BeginPaint(hWnd, &ps);
		hOldFont = (HFONT) SelectObject(hDC, GetStockObject(ANSI_FIXED_FONT));
		GetTextMetrics(hDC, &tm);

		// Display command output text
		ny = 2;
		for (i = 0; g_outputLines[i] && i < MAX_NUM_OUTPUT_LINES; i++)
		{
			TextOut(hDC, 2, ny, g_outputLines[i], strlen(g_outputLines[i]));
			ny += tm.tmHeight + tm.tmExternalLeading;
		}

		// Cleanup
		SelectObject(hDC, hOldFont);
		EndPaint(hWnd, &ps);
		return 0;

	case WM_COMMAND:

		OnCommand(hWnd, LOWORD(wParam));
		return 0;

	case WM_DESTROY:

		PostQuitMessage(0);
		return 0;

	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

/////////////////// OnCommand()
//
// Message handler for WM_COMMAND messages.

static void OnCommand(HWND hWnd, int nCmd)
{
    DiReturnT fRet;
    DiExitStatusT status;

	switch (nCmd)
	{
	case ID_FILE_EXIT:

		PostQuitMessage(0);
		break;

	case ID_MEMORY_READ:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_MEMORY_READ), hWnd,
			MemoryReadDlgProc);
		break;

	case ID_MEMORY_WRITE:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_MEMORY_WRITE), hWnd,
			MemoryWriteDlgProc);
		break;

	case ID_REGISTER_READ:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_REGISTER_READ), hWnd,
			RegisterReadDlgProc);
		break;

	case ID_REGISTER_WRITE:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_REGISTER_WRITE), hWnd,
			RegisterWriteDlgProc);
		break;

    case ID_RUN_RESET:

        fRet = DiExecResetChild();
        if (fRet != DI_OK)
            MessageBox(NULL, "Reset failed.", "Error", MB_OK|MB_ICONSTOP);
        break;

	case ID_RUN_STEP:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_RUN_STEP), hWnd,
			RunStepDlgProc);
		break;

	case ID_RUN_CONTINUE:

        fRet = DiExecContinue();
        if (fRet != DI_OK)
            MessageBox(NULL, "Continue command failed.", "Error", MB_OK|MB_ICONSTOP);
		break;

	case ID_RUN_CONTINUE_UNTIL:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_RUN_CONTINUE_UNTIL), hWnd,
			RunContinueUntilDlgProc);
		break;

    case ID_RUN_STOP:

        fRet = DiExecStop();
        if (fRet != DI_OK)
            MessageBox(NULL, "DiExecStop failed.", "Error", MB_OK|MB_ICONSTOP);
		break;

    case ID_RUN_STATUS:

        fRet = DiExecGetStatus(&status);
        if (fRet != DI_OK)
            MessageBox(NULL, "DiExecGetStatus failed.", "Error", MB_OK|MB_ICONSTOP);
        else
        {
            if (status.dscCause == DI_WAIT_RUNNING)
                MessageBox(NULL, "System is running.", "Status", MB_OK);
            else
                MessageBox(NULL, "System is stopped.", "Status", MB_OK);
        }
        break;

	case ID_BREAKPOINT_SET:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_BREAKPOINT_SET), hWnd,
			BreakpointSetDlgProc);
		break;

	case ID_BREAKPOINT_CLEAR:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_BREAKPOINT_CLEAR), hWnd,
			BreakpointClearDlgProc);
		break;

    case ID_BREAKPOINT_CLEAR_ALL:

        fRet = DiBreakpointClearAll();
        if (fRet != DI_OK)
            MessageBox(NULL, "Clear all command failed.", "Error", MB_OK|MB_ICONSTOP);
        break;

    case ID_HELP_ABOUT:

		DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_HELP_ABOUT), hWnd,
			HelpAboutDlgProc);
		break;

	default:

		break;
	}
}

/////////////////// CreateMainWindow()

static void CreateMainWindow(UINT fCmdShow)
{
	static char szClassName[] = "GdiExample";
	WNDCLASSEX wndClass;

	wndClass.cbSize			= sizeof(wndClass);
	wndClass.style			= CS_HREDRAW|CS_VREDRAW;
	wndClass.lpfnWndProc	= WndProc;
	wndClass.cbClsExtra		= 0;
	wndClass.cbWndExtra		= 0;
	wndClass.hInstance		= g_hInstance;
	wndClass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wndClass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wndClass.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
	wndClass.lpszMenuName	= MAKEINTRESOURCE(IDR_MAIN);
	wndClass.lpszClassName	= szClassName;
	wndClass.hIconSm		= LoadIcon(NULL, IDI_APPLICATION);

	RegisterClassEx(&wndClass);

	g_hMainWnd = CreateWindow(szClassName,
				"Nohau GDI Example",
				WS_OVERLAPPEDWINDOW,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				CW_USEDEFAULT,
				NULL,
				NULL,
				g_hInstance,
				NULL);

	ShowWindow(g_hMainWnd, fCmdShow);
	UpdateWindow(g_hMainWnd);
}
